/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"
`include "rtavr_common.v"

// global defines : OUTPUT_MUX_TRI

//`define OUTPUT_MUX_NONE
//`define OUTPUT_MUX_TRI

// Module Name: rtavr_ram_2KB
//  ver 005
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//

//        ___        ______        ______
//  CLK2X    |______|      |______|      |______
//
//        _____               ______________
//  CLK        |_____________|              |___
//
// CLK2X
//                   |            |
//                 read         write

module rtavr_sram # (
		parameter SIZE = 2048
//		parameter SIZE = 4096
//		parameter SIZE = 8192
) (
		input WEB,
		input [12:0] ADDRB,
		input [7:0] DIB,
		output [7:0] DOB
  `ifdef OUTPUT_MUX_NONE
  `else
		, input OEB
  `endif
`ifdef SYS_DRIVEN_CLK4PH
		, input CLK_0
		, input CLK_90
		, input CLK_180
		, input CLK_270
`else // SYS_DRIVEN_CLK2X
		, input CLK2X
		, input CLK
`endif
`ifdef RAM_HAVE_PORTA
		, input WEA
		, input [12:0] ADDRA
		, input [7:0] DIA
		, output [7:0] DOA
//  `ifdef IOR_HAVE_RAMWIN
//		, input OEA
//  `elsif OUTPUT_MUX_NONE
//  `else
//		, input OEA
//  `endif
`endif
		);

    `ifdef SYS_DRIVEN_CLK4PH
	wire CLK_90_270 = (CLK_0 & CLK_90) | (CLK_180 & CLK_270);

//	wire CLK = CLK_180;
	wire CLK_P = CLK_270;

    `else // SYS_DRIVEN_CLK2X
	wire CLK_P = CLK;
    `endif

    reg  [7:0] mem [0:SIZE-1];

    `ifdef RAM_HAVE_PORTA
	reg  [12:0] r_addra;
 
	always @(`posedge_CLK_90_270)
	begin
		if(WEA & CLK_P) mem[ADDRA] <= DIA;
		r_addra <= ADDRA;
	end
	assign DOA = mem[r_addra];
    `endif
 
    reg  [12:0] r_addrb;

    always @(`posedge_CLK_90_270)
    begin
	if(WEB & CLK_P) mem[ADDRB] <= DIB;
	r_addrb <= ADDRB;
    end

    `ifdef OUTPUT_MUX_NONE
	assign DOB = mem[r_addrb];
    `elsif OUTPUT_MUX_TRI
	bufif1(DOB, mem[r_addrb], OEB);
    `else
	assign DOB = OEB? mem[r_addrb] : 8'h00;
    `endif

`ifdef SRAM_INITCLR
  integer i;
  initial
    begin
      for (i = 0 ; i < SIZE ; i = i + 1)
	mem[i] <= 8'h00;
    end
`elsif RAM_INCLUDE_V
	`include "ram_data.v"
//initial
//  begin : ram_data
//    mem[   0] = 8'h00;
//  end
`elsif RAM_INCLUDE_DAT
  initial
    begin 
       $readmemh("ram_data.mem" , mem, 0 , SIZE-1);
    end
`endif

endmodule
