#if !defined(__linux__) && !defined(_WIN32)

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <string.h>
#include <stdio.h>

#define nop()   __asm__ volatile ("nop" )
#define wdr()   __asm__ volatile("wdr" )
#define sleep()   __asm__ volatile ("sleep" )

#else

#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <fcntl.h>

#if !defined(__linux__) 
typedef char  int8_t;
typedef short int16_t;
typedef long  int32_t;
#endif

typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned long  uint32_t;

#define nop()   
#define wdr()   
#define sleep()   
#endif


#if defined(__linux__) || defined(_WIN32)
void dump_state(int16_t *state) {
	int8_t i;
	for (i=0; i< 8; i++) {
		printf("%04x",(uint16_t)(state[i]));
	}
	printf("\n");
	for (i=0; i< 8; i++) {
		printf("%02x %02x "
			,(uint16_t)(state[i]) & 0xff
			,((uint16_t)(state[i]) >> 8) & 0xff);
	}
	printf("\n");
}
#else
 #define dump_state()
#endif

uint16_t count[8];

void shuffle_init(int16_t *state) {
	state[0] = 0x3942;
	state[1] = 0xf681;
	state[2] = 0x6122;
	state[3] = 0x380c;
	state[4] = 0x2244;
	state[5] = 0xff97;
	state[6] = 0x23a7;
	state[7] = 0xa039;
#if 0
	sleep();
#else
	nop();
#endif
}

volatile uint8_t lc = 0;

void shuffle(int16_t *state) {
	int8_t i;
	int16_t out[8];

	state[8] = state[0];
//	dump_state(state);
	for (i=0; i< 8; i++,state++) {
	  uint8_t b,c;
	  b = (*(state + 1) >> 8) & 0xff;
	  c = (*(state + 1) >> 4) & 0x7;
	  nop();
	  count[c] ++;
	  nop();
	  switch (c) {
	  case 0:
		out[i] = *state * b;
		nop();
		break;
	  case 1:
		out[i] = *state / (i+1);
		nop();
		break;
	  case 2:
		out[i] = (*state << (i+2)) | ((uint16_t)(*state) >> (14-i));
		nop();
		break;
	  case 3:
		out[i] = *state + 0x2562;
		nop();
		break;
	  case 4:
		out[i] = (*state % (16-i)) << 6;
		nop();
		break;
	  case 5:
		out[i] = *state + *(state +1);
		nop();
		break;
	  case 6:
		out[i] = (*state / (16-i)) * b;
		nop();
		break;
	  case 7:
		out[i] = *state * 3;
		nop();
		break;
	  }
	  out[i] += 0x2562;
if ((lc == 200) && (i == 1)) {
#if defined(__linux__) || defined(_WIN32)
	printf("lc %03d i %01d *state = %04x b = %02x c = %d result %04x (%04x)\n"
		,lc,i,*state & 0xffff, b, c, out[i] & 0xffff, (*state ^ out[i])&0xffff);
#else
	sleep();
#endif
}
	  nop();
	}
#if 0
	sleep();
#else
	nop();
#endif
//	dump_state(out);
	state -= 8;
	nop();
	for (i=0; i< 8; i++) {
		state[i] = state[i] ^ out[i]; 
	  	nop();
	}
#if 0
	sleep();
#else
	nop();
#endif
//	dump_state(state);
	lc ++;
}

#if defined(__linux__) || defined(_WIN32)

#ifndef LOOP_COUNT
#define LOOP_COUNT 100
#endif

int main() {
	int8_t i;
	int16_t state[9];

	shuffle_init(state);

	for (i=0; i< LOOP_COUNT ; i++) { 
		shuffle(state);
	  	nop();
	}
	/* 91a970f2f7442a4a4dca2e464b62dd4c */
	/* 16 loop :
	 	5b4451aeb62f0ac8fd0a494986751e02
	  	12   16   14   12   31    6   19   18 
	*/

	dump_state(state);
	for (i=0; i< 8; i++) {
		printf("%4d ",count[i]);
	}
	printf("\n");
	return 0;
}
#endif
