`include "rtavr_defs.v"
`include "rtavr_common.v"

`ifdef RTAVR_TARGET_MachXO
   `define JTAG_BSCAN JTAGD
   `define HAVE_OSCC
//	module OSCC (OSC); //synthesis syn_black_box
//	  output OSC;
//	endmodule

`elsif RTAVR_TARGET_MachXO2
   `define JTAG_BSCAN JTAGF

   `define HAVE_OSCH
//	module OSCH (
//	   input    STDBY,
//	   output   OSC, SEDSTDBY ); //synthesis syn_black_box
//
//	   parameter NOM_FREQ = "2.08";
//	endmodule
`endif

module xo2_isp (
		// JTAG 
		input TOP_TDI
		, input TOP_TCK
		, input TOP_TMS
		, output  TOP_TDO

		// LED
		,  output [7:0] LED
`ifdef RTAVR_TARGET_MachXO2
		// Serial
		, input TXD, RTS, DTR
		, inout RXD, CTS
		// External OSC
		, input EXTOSC
		, inout EXTOSC_EN
`endif
		);

  `JTAG_BSCAN bscan (
`ifdef RTAVR_TARGET_MachXO
	.JRST(BSCAN_RESET) 
`elsif RTAVR_TARGET_MachXO2
	.JRSTN(BSCAN_RESET_N) 
`endif
	, .JSHIFT(BSCAN_SHIFT)
//	, .JUPDATE(BSCAN_UPDATE)

	, .TDI(TOP_TDI)
	, .TCK(TOP_TCK)
	, .TMS(TOP_TMS) 
	, .TDO(TOP_TDO)

	, .JTDI(JTDI_IN)
	, .JTCK(JTCK_IN) // negedge capt, posedge shift

	// signals for IPA
	, .JRTI1(JRTI1)
	, .JCE1(SEL1)
	, .JTDO1(JTDO1)

	// signals for IPB
	, .JRTI2(JRTI2)
	, .JCE2(SEL2)
	, .JTDO2(JTDO2) 
	);

`ifdef RTAVR_TARGET_MachXO
`elsif RTAVR_TARGET_MachXO2
    wire BSCAN_RESET = ~BSCAN_RESET_N;
`endif

`ifdef HAVE_OSCC
  OSCC osc_internal (.OSC(CLK_INT)); // 18 MHz - 26 MHz
`elsif HAVE_OSCH
  OSCH #(.NOM_FREQ("24.18") ) osc_internal
	(.OSC(CLK_INT), .STDBY(1'b0));
`endif

`ifdef SYS_DRIVEN_CLK4PH
`else
  wire CLK_IN;
`endif

`ifdef RTAVR_TARGET_MachXO2
//`ifdef CLK_SELECTOR

  wire CLK_IN0;

//  DELAYE #(.DEL_VALUE("DELAY31")) crystal_drv 
//	( .A(EXTOSC), .Z(EXTOSC_D));
  `ifdef USE_EXTOSC
     assign EXTOSC_EN = 1'b1;
  `else
     assign EXTOSC_EN = ~EXTOSC;
  `endif

  DCMA clk_selector (.CLK0(CLK_INT) 
	, .CLK1(EXTOSC)
	, .SEL(1'b1) // 0: CLK0 1: CLK1
	, .DCMOUT(CLK_IN0)
	);

  EHXPLLJ #(
	.CLKI_DIV(24)
	, .CLKFB_DIV(33)
	, .CLKOP_DIV(16)
	, .CLKOS_DIV(16)
	, .CLKOS2_DIV(16)
	, .CLKOS3_DIV(16)
	, .CLKOP_FPHASE(0)
	, .CLKOS_FPHASE(2)
	, .CLKOS2_FPHASE(4)
	, .CLKOS3_FPHASE(6)
	) pll
	(.CLKI(CLK_IN0)
  `ifdef SYS_DRIVEN_CLK4PH
    `ifdef USE_DMY_CLOCK
		, .CLKOP(CLK_IN)
    `else
		, .CLKOP(CLK0_OUT)
		, .CLKOS(CLK90_OUT)
		, .CLKOS2(CLK180_OUT)
		, .CLKOS3(CLK270_OUT)
		, .ENCLKOS(1'b1)
		, .ENCLKOS2(1'b1)
		, .ENCLKOS3(1'b1)
    `endif
  `else // SYS_DRIVEN_CLK2X
		, .CLKOP(CLK_IN)
  `endif
	);
`else
  assign EXTOSC_EN = 1'bz;
  assign CLK_IN = CLK_INT;
`endif


`ifdef SYS_DRIVEN_CLK4PH
  `ifdef USE_DMY_CLOCK
	assign CLKFX = CLK_IN;
	assign CLK = CLK_OUT;
	assign RESET = ~BSCAN_RESET;
  `else
	assign CLK_0   = CLK0_OUT;
	assign CLK_90  = CLK90_OUT;
	assign CLK_180 = CLK180_OUT;
	assign CLK_270 = CLK270_OUT;
	assign CLK = CLK_180;
	assign RESET = ~BSCAN_RESET;
  `endif
`else // SYS_DRIVEN_CLK2X
	assign CLKFX = CLK_IN;
	assign CLK = CLK_OUT;
	assign RESET = ~BSCAN_RESET;
`endif
 
  reg r_jtck;
  reg r_jtck_prev;
  reg r_cs1;
  reg r_cs1_prev;
  reg r_cs2;
  reg r_cs2_prev;
  always @(posedge CLK)
    begin
	r_jtck_prev <= JTCK_IN;
	r_cs1_prev   <= SEL1 & BSCAN_SHIFT;
	r_cs2_prev   <= SEL2 & BSCAN_SHIFT;
	if (r_jtck_prev == JTCK_IN)
		r_jtck <= r_jtck_prev;
	if (r_cs1_prev == (SEL1 & BSCAN_SHIFT))
		r_cs1   <= r_cs1_prev;
	if (r_cs2_prev == (SEL2 & BSCAN_SHIFT))
		r_cs2   <= r_cs2_prev;
    end
  wire JTCK1 = ~r_jtck;
  wire JTCK2 = ~r_jtck;
  wire JTDI  = JTDI_IN;
  wire CS1 = ~r_cs1;
  wire CS2 = ~r_cs2;

`ifdef SUPPORT_ISP
    wire [15:0] ISP_ADDR;
    wire [7:0]  ISP_STORE_DATA;
    wire [7:0]  ISP_LOAD_DATA;
  isp ISP (.CLK(CLK)
		, .MOSI(JTDI)
                , .SCK(JTCK2)
                , .CS(CS2)
                , .MISO(JTDO2)

                , .ISP_STORE(ISP_STORE)
                , .ISP_LOAD(ISP_LOAD)
                , .ISP_ADDR(ISP_ADDR)
                , .ISP_STORE_DATA(ISP_STORE_DATA)
                , .ISP_LOAD_DATA(ISP_LOAD_DATA)
		);
`endif

  `ifdef RTAVR_PORT_ESCALATION
    wire [23:0] DDR;
    wire [23:0] PORT;
    wire [23:0] PIN;
  `endif
  
  rtavr RTAVR (.RESET(RESET)
  `ifdef SYS_DRIVEN_CLK4PH
     `ifdef USE_DMY_CLOCK
		, .CLK4X(CLKFX)
		, .CLK_OUT(CLK_OUT)
     `else
		, .CLK_0(CLK0_OUT)
		, .CLK_90(CLK90_OUT)
		, .CLK_180(CLK180_OUT)
		, .CLK_270(CLK270_OUT)
     `endif
  `else // SYS_DRIVEN_CLK2X
		, .CLK2X(CLKFX)
		, .CLK_OUT(CLK_OUT)
  `endif

  `ifdef RTAVR_PORT_ESCALATION
		, .DDR(DDR[23:0])
		, .PORT(PORT[23:0])
		, .PIN(PIN[23:0])
  `else
    `ifdef IOR_HAVE_PORTA
		, .PORTA(PORTA[7:0])
    `endif
    `ifdef IOR_HAVE_PORTB
		, .PORTB(PORTB[7:0])
    `endif
    `ifdef IOR_HAVE_PORTC
		, .PORTC(PORTC[7:0])
    `endif
  `endif
`ifdef SUPPORT_ISP
	,  .ISP_STORE(ISP_STORE)
	,  .ISP_LOAD(ISP_LOAD)
	,  .ISP_ADDR(ISP_ADDR[15:0])
	,  .ISP_STORE_DATA(ISP_STORE_DATA[7:0])
	,  .ISP_LOAD_DATA(ISP_LOAD_DATA[7:0])
`endif
 );

//`define	 SS_BIT    16     // PC0
//`define	 SCK_BIT   17     // PC1
//`define	 MISO_BIT  18     // PC2
//`define	 MOSI_BIT  20     // PC4
//`define	 RX_BIT    21     // PC5
//`define	 TX_BIT    22     // PC6
// PC3 (19): -> VS0
// PC7 (23): -> VS2
`ifdef RTAVR_TARGET_MachXO2
  assign CTS = 1'bz;
    `ifdef IOR_HAVE_USART
    `else
	assign RXD = 1'bz;
    `endif
`endif
  assign LED[2] = 1'b1;
  assign LED[3] = 1'b1;
  assign LED[4] = 1'b1;
  assign LED[5] = 1'b1;
  assign LED[6] = 1'b1;
  assign LED[7] = 1'b1;

  `ifdef RTAVR_PORT_ESCALATION
	assign LED[0] = DDR[19] ? PORT[19] : "H";
	assign PIN[19] = LED[0];
	assign LED[1] = DDR[23] ? PORT[23] : "H";
	assign PIN[23] = LED[1];
    `ifdef IOR_HAVE_SPI
	assign PIN[`SS_BIT]   = CS1;
	assign PIN[`SCK_BIT]  = JTCK1;
	assign PIN[`MOSI_BIT] = JTDI;
	assign JTDO1 = PORT[`MOSI_BIT];
    `endif
    `ifdef IOR_HAVE_USART
	assign PIN[`RX_BIT]   = TXD;
	assign RXD = DDR[`TX_BIT] ? PORT[`TX_BIT] : "H";
	assign PIN[`TX_BIT]   = RXD;
    `endif 
  `endif 

endmodule
