#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <unistd.h>

#include "util.h"
#include "cable.h"
#include "serjtag.h"
#include "ft245r.h"
#include "jtag.h"

#include "target.h"
#include "tgtlscc.h"
#include "tgtxil.h"

extern char *program;
extern char *progname;
extern int verbose;

void jtag_test(CABLE cbl)
{
	uint32_t r;
	double st,et;

	st = dtime();
	usleep(200000);
	et = dtime();
	printf("usleep 200ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

    if (0 && cbl->delay) {
	st = dtime();
	(cbl->delay)(cbl, 4000);
	et = dtime();
	printf("delay 4ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 9000);
	et = dtime();
	printf("delay 9ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 20000);
	et = dtime();
	printf("delay 20ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 200000);
	et = dtime();
	printf("delay 200ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 400000);
	et = dtime();
	printf("delay 400ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	(cbl->delay)(cbl, 600000);
	et = dtime();
	printf("delay 600ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);
    }

	jtag_init(cbl);

	st = dtime();
	jtag_IDLE(cbl, 20, 5);
	et = dtime();
	printf("jtag_IDLE 20ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	jtag_IDLE(cbl, 20, 5);
	et = dtime();
	printf("jtag_IDLE 20ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	st = dtime();
	jtag_IDLE(cbl, 200, 5);
	et = dtime();
	printf("jtag_IDLE 200ms = %7.3f ms\n", (et - st) * 1000.0);
	fflush(stdout);

	r = 0;
//verbose = 256;
/*
 * ! Shift in IDCODE(0x16) instruction
 * SIR     8       TDI  (16);
 * SDR     32      TDI  (FFFFFFFF)
 *                 TDO  (0128D043)
 *                 MASK (FFFFFFFF);
 */
	jtag_SIR32(cbl, LSCC_IDCODE, 8, 1, &r);
	printf ("jtag idcode I = %08x\n" ,r);
fflush(stdout);
	jtag_SDR32(cbl, 0xffffffff, 32, 1, &r);
	printf ("jtag idcode D = %08x\n" ,r);

	jtag_IDLE(cbl, 1, 5);
//verbose = 256;
	jtag_SIR32(cbl, LSCC_IDCODE, 8, 1, &r);
	printf ("jtag idcode I = %08x\n" ,r);
fflush(stdout);
	jtag_SDR32(cbl, 0xffffffff, 32, 1, &r);
	printf ("jtag idcode D = %08x\n" ,r);
verbose = 1;
/* 
 * ! Shift in READ USERCODE(0x17) instruction
 * SIR     8       TDI  (17);
 * RUNTEST IDLE    5 TCK   1.00E-003 SEC;
 * SDR     32      TDI  (FFFFFFFF)
 *                 TDO  (FFFFFFFF)
 *                 MASK (00000000);
 */
	jtag_SIR32(cbl, LSCC_USERCODE, 8, 1, &r);
	printf ("jtag usercode I = %08x\n" ,r);
	jtag_IDLE(cbl, 1, 5);
	jtag_SDR32(cbl, 0xffffffff, 32, 1, &r);
	printf ("jtag usercode D = %08x\n" ,r);
	jtag_IDLE(cbl, 1, 5);
}

void spi_test(TARGET tgt, CABLE cbl)
{
	int i,j;
	uint32_t s,r;
	printf ("spi_test -- sel_chan test\n");
	fflush(stdout);
	for (i=0; i< 4; i++) {
		jtag_reset(cbl);
		(tgt->sel_chan)(tgt, cbl, (i&1)?1:2);
		for (j=0; j< 5; j++) {
			usleep(100000);
		}
	}
	printf ("spi_test -- spi_echo test1\n");
	fflush(stdout);
	jtag_reset(cbl);
	(tgt->sel_chan)(tgt, cbl, 1);
	for (i=0; i< 12; i++) {
		s = (i<10)?(('0'+i) & 0xff):0xff;
		if (i==5) s |= 0x5500;
		if (i==6) s |= 0xaa5500;
//if ((i==5) || (i==6)) verbose = 256;
		jtag_SDR32R(cbl, s , (i==6)?24:(i==5)?16:8, 1, &r);
		printf ("  %02x %02x (%02x)\n" , s, r, r>>1);

verbose = 1;
		fflush(stderr);
		fflush(stdout);
		for (j=0; j< 2; j++) {
			usleep(100000);
		}
	}
	jtag_reset(cbl);
	printf ("spi_test done\n");
	fflush(stdout);
}
