//`define TARGET_MachXO2
`define TARGET_MachXO
//`define SYS_DRIVEN_CLK4PH
//`define USE_EXTOSC

`ifdef TARGET_MachXO
   `define JTAG_BSCAN JTAGD
   `define HAVE_OSCC
//	module OSCC (OSC); //synthesis syn_black_box
//	  output OSC;
//	endmodule

`elsif TARGET_MachXO2
   `define JTAG_BSCAN JTAGF

   `define HAVE_OSCH
//	module OSCH (
//	   input    STDBY,
//	   output   OSC, SEDSTDBY ); //synthesis syn_black_box
//
//	   parameter NOM_FREQ = "2.08";
//	endmodule
`endif

module tool_test (
		// JTAG 
		  input TOP_TDI
		, input TOP_TCK
		, input TOP_TMS
		, output  TOP_TDO

		// LED
		,  output [7:0] LED
`ifdef TARGET_MachXO2
		// Serial
		, input TXD, RTS, DTR
		, inout RXD, CTS
		// External OSC
		, input EXTOSC
		, inout EXTOSC_EN
`endif
		);

`ifdef TARGET_MachXO2
  wire RXD = 1'bz;
  wire CTS = 1'bz;
  wire EXTOSC_EN = 1'bz;
`endif

  `JTAG_BSCAN bscan (
`ifdef TARGET_MachXO
	.JRST(BSCAN_RESET) 
`elsif TARGET_MachXO2
	.JRSTN(BSCAN_RESET_N) 
`endif
	, .JSHIFT(BSCAN_SHIFT)
//	, .JUPDATE(BSCAN_UPDATE)

	, .TDI(TOP_TDI)
	, .TCK(TOP_TCK)
	, .TMS(TOP_TMS) 
	, .TDO(TOP_TDO)

	, .JTDI(JTDI_IN)
	, .JTCK(JTCK_IN) // negedge capt, posedge shift

	// signals for IPA
	, .JRTI1(JRTI1)
	, .JCE1(SEL1)
	, .JTDO1(JTDO1)

	// signals for IPB
	, .JRTI2(JRTI2)
	, .JCE2(SEL2)
	, .JTDO2(JTDO2) 
	);

`ifdef TARGET_MachXO
`elsif TARGET_MachXO2
    wire BSCAN_RESET = ~BSCAN_RESET_N;
`endif

  wire CLK;

`define JTAG_TEST3
`ifdef JTAG_TEST3
  reg r_jtck;
  reg r_jtck_prev;
  reg r_cs1;
  reg r_cs1_prev;
  reg r_cs2;
  reg r_cs2_prev;
  always @(posedge CLK)
    begin
	r_jtck_prev <= JTCK_IN;
	r_cs1_prev   <= SEL1 & BSCAN_SHIFT;
	r_cs2_prev   <= SEL2 & BSCAN_SHIFT;
	if (r_jtck_prev == JTCK_IN)
		r_jtck <= r_jtck_prev;
	if (r_cs1_prev == (SEL1 & BSCAN_SHIFT))
		r_cs1   <= r_cs1_prev;
	if (r_cs2_prev == (SEL2 & BSCAN_SHIFT))
		r_cs2   <= r_cs2_prev;
    end
  wire JTCK1 = ~r_jtck;
  wire JTCK2 = ~r_jtck;
  wire JTDI  = JTDI_IN;
  wire CS1 = ~r_cs1;
  wire CS2 = ~r_cs2;
`elsif JTAG_TEST2
  reg r_jtck;
  reg r_jtck_prev;
  reg r_jtdi;
  reg r_jtdi_prev;
  reg r_cs1;
  reg r_cs1_prev;
  reg r_cs2;
  reg r_cs2_prev;
  always @(posedge CLK)
    begin
	r_jtck_prev <= JTCK_IN;
	r_jtdi_prev <= JTDI_IN;
	r_cs1_prev   <= SEL1 & BSCAN_SHIFT;
	r_cs2_prev   <= SEL2 & BSCAN_SHIFT;
	if (r_jtck_prev == JTCK_IN)
		r_jtck <= r_jtck_prev;
	if (r_jtdi_prev == JTDI_IN)
		r_jtdi <= r_jtdi_prev;
	if (r_cs1_prev == (SEL1 & BSCAN_SHIFT))
		r_cs1   <= r_cs1_prev;
	if (r_cs2_prev == (SEL2 & BSCAN_SHIFT))
		r_cs2   <= r_cs2_prev;
    end
  wire JTCK1 = ~r_jtck;
  wire JTCK2 = ~r_jtck;
  wire JTDI  = r_jtdi;
  wire CS1 = ~r_cs1;
  wire CS2 = ~r_cs2;
`else // JTAG_TEST1
  wire JTCK1 = JTCK_IN;
  wire JTCK2 = JTCK_IN;
  wire JTDI  = JTDI_IN;
  wire CS1 = ~(SEL1 & BSCAN_SHIFT);
  wire CS2 = ~(SEL2 & BSCAN_SHIFT);
`endif
  wire [7:0] SPI_DATA;

  spi_echo SPI_ECHO (.CLK(CLK)
		, .MOSI(JTDI)
                , .SCK(JTCK1)
                , .CS(CS1)
                , .MISO(JTDO1)
		, .SPI_DATA(SPI_DATA)
		);
 
  isp ISP (.CLK(CLK)
		, .MOSI(JTDI)
                , .SCK(JTCK2)
                , .CS(CS2)
                , .MISO(JTDO2)
		);

  wire CLK_INT;

`ifdef HAVE_OSCC
  OSCC osc_internal (.OSC(CLK_INT)); // 18 MHz - 26 MHz
`elsif HAVE_OSCH
  OSCH #(.NOM_FREQ("24.18")) osc_internal
	(.OSC(CLK_INT), .STDBY(1'b0));
`endif

`ifdef TARGET_MachXO2

  `ifdef USE_EXTOSC
     assign EXTOSC_EN = 1'b1;
  `else
     assign EXTOSC_EN = ~EXTOSC;
  `endif

  DCMA clk_selector (.CLK0(CLK_INT) 
	, .CLK1(EXTOSC)
	, .SEL(1'b0) // 0: CLK0 1: CLK1
	, .DCMOUT(CLK_IN0)
	);

  EHXPLLJ #(
	.CLKI_DIV(24)
	, .CLKFB_DIV(24)
	, .CLKOP_DIV(16)
	, .CLKOS_DIV(16)
	, .CLKOS2_DIV(16)
	, .CLKOS3_DIV(16)
	, .CLKOP_FPHASE(0)
	, .CLKOS_FPHASE(2)
	, .CLKOS2_FPHASE(4)
	, .CLKOS3_FPHASE(6)
	) pll
	(.CLKI(CLK_IN0)
  `ifdef SYS_DRIVEN_CLK4PH
    `ifdef USE_DMY_CLOCK
		, .CLKOP(CLK_IN)
    `else
		, .CLKOP(CLK0_OUT)
		, .CLKOS(CLK90_OUT)
		, .CLKOS2(CLK180_OUT)
		, .CLKOS3(CLK270_OUT)
		, .ENCLKOS(1'b1)
		, .ENCLKOS2(1'b1)
		, .ENCLKOS3(1'b1)
    `endif
  `else // SYS_DRIVEN_CLK2X
		, .CLKOP(CLK_IN)
  `endif
	);
`else
  assign EXTOSC_EN = 1'bz;
  assign CLK_IN = CLK_INT;
`endif

`ifdef SYS_DRIVEN_CLK4PH
  wire CLK = CLK_INT;
//  wire CLK = CLK0_OUT;
//  wire CLK = CLK90_OUT;
//  wire CLK = CLK180_OUT;
//  wire CLK = CLK270_OUT;
`else
  wire CLK = CLK_INT;
`endif

//  reg [24:0] clk_count;
  reg [26:0] clk_count;
  
  always @(posedge CLK)
    begin
         clk_count = clk_count + 1;
    end

//`define CLK_TEST
`define TEST_02
`ifdef CLK_TEST
  reg [26:0] clk_count0;
  reg [26:0] clk_count1;
  
  always @(posedge CLK_IN0)
    begin
         clk_count0 = clk_count0 + 1;
    end

`ifdef SYS_DRIVEN_CLK4PH
  always @(posedge CLK0_OUT)
    begin
         clk_count1 = clk_count1 + 1;
    end
  reg [26:0] clk_count2;
  reg [26:0] clk_count3;
  reg [26:0] clk_count4;
  always @(posedge CLK90_OUT)
    begin
         clk_count2 = clk_count2 + 1;
    end
  always @(posedge CLK180_OUT)
    begin
         clk_count3 = clk_count3 + 1;
    end
  always @(posedge CLK270_OUT)
    begin
         clk_count4 = clk_count4 + 1;
    end
`else
  always @(posedge CLK_IN)
    begin
         clk_count1 = clk_count1 + 1;
    end
`endif


  assign LED[7] = ~clk_count[24];
  assign LED[6] = ~clk_count0[24];
`ifdef SYS_DRIVEN_CLK4PH
  assign LED[5] = ~clk_count1[24];
  assign LED[4] = ~clk_count2[24];
  assign LED[3] = ~clk_count3[24];
  assign LED[2] = ~clk_count4[24];
`else
  assign LED[5] = ~clk_count1[24];
  assign LED[4] = ~SPI_DATA[4];
  assign LED[3] = ~SPI_DATA[3];
  assign LED[2] = ~SPI_DATA[2];
`endif
  assign LED[1] = ~SPI_DATA[1];
  assign LED[0] = ~SPI_DATA[0];

`elsif TEST_02
  assign LED[7] = ~clk_count[24];
  assign LED[6] = CS2;
  assign LED[5] = CS1;
  assign LED[4] = ~SPI_DATA[4];
  assign LED[3] = ~SPI_DATA[3];
  assign LED[2] = ~SPI_DATA[2];
  assign LED[1] = ~SPI_DATA[1];
  assign LED[0] = ~SPI_DATA[0];
`else // TEST_01
  // 1st : check led and internal clock
  assign LED[0] = ~(clk_count[26:24] == 0);
  assign LED[1] = ~(clk_count[26:24] == 1);
  assign LED[2] = ~(clk_count[26:24] == 2);
  assign LED[3] = ~(clk_count[26:24] == 3);
  assign LED[4] = ~(clk_count[26:24] == 4);
  assign LED[5] = ~(clk_count[26:24] == 5);
  assign LED[6] = ~(clk_count[26:24] == 6);
  assign LED[7] = ~(clk_count[26:24] == 7);
`endif
endmodule
